%% running FelMuG Algorithm for Graph data
% max_{d} min{w} 1/2 \sum_1^T |w_t|^2 + C \sum_{t,i}\xi_{t,i}
%  s.t. y_{t,i}w_t^T * (x_{t,i}\odot (d+d_ts+d_ta))+\xi_{t,i} >=1;
%  common feature and task specified features is used, i.e., $d$ represents
%  the common feature $d_t$ represents the task specific features $d \odot d_t =0$
function [model,accs] = FelMuG_graph(X,Y,opt, Xte,Yte)
% Input: X, Y -- Cell for multi-task training data,graph data
%        Opt -- Configuration for multi-task learning
%        Xte, Yte -- Cell for multi-task testing data, optimal, if
%        provided, evaluate the accuracy for each iteration
%
% Output: model -- the learned model
%         accs -- the accuracy on testing data if testing data is provided
%
%

import util.*;
import moss.*;
K=opt.K; %Least Number of Features for Multi-task learning
Kt=opt.Kt; %task specific features
Ka=opt.Ka; %task auxiliary features
epslog= opt.epslog;%5*1e-3; %stopping condition for outer iterations
C=opt.C; %K=0.5
normalizeKernels=opt.normalizeKernels;
maxInt = opt.maxInt;
debug_msg = opt.debug_msg; % if set, output additional message for debugging
equalmu = opt.equalmu;



test = 0;
if nargin >3  % test data is provided!
    test = 1;
end


%% Setting MKMT solver options

%------------------------------------------------------
% choosing the stopping criterion
%------------------------------------------------------
options.stopvariation=1; % use variation of weights for stopping criterion
options.stopKKT=1;       % set to 1 if you use KKTcondition for stopping criterion


%------------------------------------------------------
% choosing the stopping criterion value
%------------------------------------------------------
options.seuildiffsigma=1e-5;        % stopping criterion for weight variation
options.seuildiffconstraint=0.1;    % stopping criterion for KKT


%------------------------------------------------------
% Setting some numerical parameters
%------------------------------------------------------
options.goldensearch_deltmax=1e-1; % initial precision of golden section search
options.numericalprecision=1e-8;   % numerical precision weights below this value
% are set to zero
options.lambdareg = 1e-8;          % ridge added to kernel matrix

%------------------------------------------------------
% some algorithms paramaters
%------------------------------------------------------
options.firstbasevariable='first'; % tie breaking method for choosing the base
% variable in the reduced gradient method
options.nbitermax=100;             % maximal number of iteration
options.seuil=0;                   % forcing to zero weights lower than this
options.seuilitermax=10;           % value, for iterations lower than this one

options.miniter=0;                 % minimal number of iterations
options.verbosesvm=0;              % verbosity of inner svm algorithm
options.multiclass=0;


%--------------------------------------------------------------------------
% p norm parameter
%--------------------------------------------------------------------------
options.stopvariationlp=1; % use variation of weights for stopping criterion
options.seuildiffsigmalp=1e-2;
options.linear=0;

options.lpnbitermax = 100;
options.norm_p  = 1;
options.epsilon=1;
options.linear=0;


%% Cutting plane training of common feature multi-task svm
T=length(X);
%Initial alpha
alpha=cell(T,1);
sumalpha=zeros(T,1);
totalIns=0;
for i=1:T
    [nt]=size(X{i});
    yp=sum(Y{i}==1);
    
    yn=sum(Y{i}==-1);
    alpha{i}= zeros(nt,1);
    alpha{i}(Y{i}==1)=C*yn/yp;
    alpha{i}(Y{i}==-1)=C*yp/yn;
    

    sumalpha(i)=sum(alpha{i});
    totalIns=totalIns + sumalpha(i);
end
Obj_0=totalIns;
prtmsg(['Initial Objective Values:',num2str(Obj_0)],debug_msg)




omega=Obj_0; %Initialize all examples is wrongly classified
Obj_old=Obj_0;

KM=cell(1,T); %Kernels for multi-task
for t=1:T
    KM{t}=zeros(0,0,0);
end

inter = 1;
while true

    prtmsg(['Begin interation:',num2str(inter)],debug_msg)
    %most violated feature selection
    [d,d_t,score,Fea,subgraphs,d_a]=mostViolatedFea_GroupGraph(X,Y,alpha,K,Kt,Ka,opt);
    
    F_ad=-0.5*score +sum(sumalpha);
    
     prtmsg(['F_ad:',num2str(F_ad),'  MTMK value:', num2str(omega)],debug_msg);
     if F_ad  >= omega +epslog 
         disp('Convergence because F_ad >= omega + epslog')
         break;
     end
     
    WC(inter).comIndex=d;
    WC(inter).taskIndex=d_t; %add newly task specific features
    WC(inter).auxIndex=d_a;
    WC(inter).subgraphs=subgraphs;
    WC(inter).F=Fea;
    disp(['No of Subgraph Used:',num2str(subgraphs.size)])

    
    %% Solve multiple kernel multi-task algorithm
    %add the new kernel
    for t=1:T
        K_temp=KM{t};
        exsize=size(K_temp,3);
        KM{t}(:,:,exsize+1)=Fea{t}*Fea{t}';%K_temp;
    end
    
    if normalizeKernels==1
    %Normalize Kernel
     [KM,Weigths]=mtlNormalizekernels(KM);
    else
     Weigths=cell(1,T); %instep of normalize kernels, Weight for each kernel is 1
     for i=1:T
         nbk=size(KM{i},3);
         Weigths{i}=ones(nbk,1);
     end
    end



    verbose=1;

    prtmsg(['Solving MKMT problem,number of active constraint:',num2str(size(WC,1))],debug_msg)


    options.alphainit=alpha; % warm start for fast training
     
     %                  EM L1-Lq, solving multi-kernel multi-task SVM
    %--------------------------------------------------------------------------
    options.norm_q=0;        
    linear=1;
    
    %run once
    if equalmu %make the mu for different tasks has equal weights 
        [mu,w,bsvm,pos,Norme,storyEM]=mtlmklsvmEM(KM,Y,C,options,verbose);
    else
        storyEM.obj = 0;
        nbk=size(KM{1},3);

        mu=zeros(T,nbk);
        sumobj = 0;
        for t=1:T
        %run T-time
            newopt=options;
            newopt.alphainit=alpha(t);
    
            [mu(t,:),w(t),bsvm(t),pos(t),~,stm]=mtlmklsvmEM(KM(t),Y(t),C,newopt,verbose);
            sumobj = sumobj + stm.obj(end);
        end
        storyEM.obj = sumobj;        
        
    end

    % record the result
    model.normKernelWeight = Weigths;
    model.mu = mu;
    model.w=w;
    model.bsvm = bsvm;
    model.pos = pos;
    model.linear = linear;
    model.WC = WC;

    
    if test==1 %test data is provided    
        [ErrL1L2,ypred,AUCL1L2,accs]=EvalMultiTask_Graph(Xte,Yte,model);
        prtmsg(['ACC:',num2str(1-ErrL1L2),' AUC:',num2str(AUCL1L2)],debug_msg)
    end
    %organize the selected features
%     d_bar=mu(1,:) * WC;
      
     omega=storyEM.obj(length(storyEM.obj));
     prtmsg(['Objective Values:',num2str(omega)],debug_msg);
    
     %relativeValue = ( Obj_old - omega)/Obj_0;
     relativeValue = ( Obj_old - omega)/Obj_old;
     prtmsg(['Relative objective value:',num2str(relativeValue)],debug_msg)
     prtmsg('===================================================',debug_msg)
% 
     if relativeValue < epslog
         prtmsg(['Convergence because relative value is smaller than: ',num2str(epslog)],debug_msg);
         break;
     end
     
     for t=1:T        
        [nt]=length(Y{t});
        Y_t=Y{t};
        newalpha=zeros(nt,1);
        newalpha(pos{t})=w{t} .* Y_t(pos{t});
        newalpha=sparse(newalpha);
        alpha{t}= newalpha;
        sumalpha(t)=sum(alpha{t});
     end
          
     Obj_old = omega;
     if inter==maxInt;
          break;
     end
     inter=inter+1;

end
   
end






function stms_array = TopKsubgraphMining(G,alpha, supp, no_stumps, maxPad, noTrain)

 import util.*;
  import moss.*;
  ext = java.util.ArrayList;
  GraphUtils.setWeights(G,alpha); 
  
  gminer = GraphBoostMinerGraphsMtask;
  squareScore = true;
  %supp=0; %do not set a support for MTG
  stms = gminer.getMaxGainDecisionStumps(G,ext, supp, 0, no_stumps, maxPad, 0, noTrain,squareScore);


  stms_array = GraphUtils.getStumpsFromTreeset(stms);
end

%finding the most violated feature via subgraph mining and solving the binary linear problem 
function [d,d_t,value,Fea,allstumps,d_a] = mostViolatedFea_GroupGraph(G,Y,alpha,K,Kt,Ka,opt)

import util.*;
import moss.*; 
%javasetting;
 
T=length(G); % number task
% First step, mine top K+sum K_t subgraphs for each task
allstumps = java.util.ArrayList;

X=cell(T,1);
no_stumps = K + sum(Kt)+sum(Ka);
%all = java.util.ArrayList;
all=cell(T,1);
supp = opt.supp;
maxPad=opt.maxPad;
for t = 1 : T
   %no_stumps = K + Kt(t)+Ka(t);
   noTrain = length(Y{t});
   graphs=G{t};

   all{t} = TopKsubgraphMining(graphs,alpha{t}, supp,  no_stumps, maxPad, noTrain);
   %disp(['Number of subgraphs:',num2str(all{t}.size)])       
end
%disp('over')
for t=1:T
    allstumps = GraphUtils.mergeTwoGraphSet(allstumps,all{t});
end
disp(['Number of stumps:',num2str(allstumps.size)])

%disp(['Nb of potential stumps:',num2str(allstumps.size)])

%Second, transfer graph into vectors
for t=1:T
    X{t}=GraphUtils.graph2Vector(G{t},allstumps);
end




% the top K features over all tasks are common feature

m=size(X{1},2); %number of features
%disp(['XXXX Features:',num2str(m)])
s_t=zeros(T,m); % feature score for each feature for one task
Fea=cell(T,1);
for k=1: m   
    for t=1:T
        X_t = X{t};
        score=(alpha{t} .* Y{t})' * X_t(:,k);
        s_t(t,k) = score * score;
    end   
end



s=sum(s_t,1); %sum of the feature score over all tasks



%A feature alignment: common feature, task specificed feature, task
%auxiliary feature

totalF=m*T;
single_s = reshape(s_t',1,totalF);
f = [s single_s single_s];
A1=[ones(1,m), zeros(1,2*totalF)]; %constraint for common feature
b1=K;
A=A1;
for t=1:T
    A2=zeros(1,m+2*totalF);
    A2(t*m+1:t*m+m)=1; %constraint for task specific feature, |d_t|<=K_t
    A=[A;A2];
end
b2=Kt;
b=[b1;b2];

for t=1:T
    A4=zeros(1,m+2*totalF);
    A4(totalF+t*m+1:totalF+t*m+m)=1; %constraint for task auxiliary feature, |d_a|<=K_a
    A=[A;A4];
end
b4=Ka;
b=[b;b4];

%task specific feature is unique feature, the second part
A5=zeros(m,m+2*totalF);
for j=1:m     
     for t =1:T
         %A5(j,totalF+t*m+j)=1;  %constrain for task specific feature \sum_a=1^T d^a_j
         A5(j,t*m+j)=1;  %constrain for task specific feature \sum_a=1^T d^a_j
     end
end
A=[A;A5];
b=[b;ones(m,1)];



for t=1:T
    A3=zeros(m,m+2*totalF);
    for i=1:m
        A3(i,i)=1;
        A3(i,t*m+i)=1;     %for each task, d_j + d^t_j +d^a_j<=1;
        A3(i,totalF+t*m+i)=1;
    end
    A=[A;A3];
    b=[b;ones(m,1)];
end

ops.Display = 'off';

[xs,FVAL] = bintprog(-f,A,b,[],[],[],ops);

value = -FVAL;
d1=xs(1:m)'; %First is common Features
dx=xs(m+1:m+totalF); %Task specific features
dy=xs(m+totalF+1:end); %Task auxiliary Features
d_t1=reshape(dx,m,T)';
d_a1=reshape(dy,m,T)';

m=size(X{1},2); %number of features
%d=zeros(1,m);
d_t=zeros(T,m);
d_a=zeros(T,m);
%dI = d1==1;
%dI = PI(d1==1);
%disp(['COMMON:',num2str(dI')])
%d(dI)=1;
d=d1;
for t=1:T
    %dI1=PI(d_t1(t,:)==1);
    dI1=(d_t1(t,:)==1);
    d_t(t,dI1)=1;
    
    dI2=(d_a1(t,:)==1);
    %dI2=PI(d_a1(t,:)==1);
    d_a(t,dI2)=1;
end


    
for t=1:T
    X_t=X{t};
    d_s = d + d_t(t,:)+d_a(t,:); %final selected features
    ind = d_s>1e-5;
    Fea{t}=X_t(:,ind);
end



end


function prtmsg(msg,debug)
    if nargin<2 || debug == 1; 
        disp(msg)
    end
end

