% Example of computational complexity comparison  
%
%

clear all
close all
addpath('./grad/')




r=[4];
N=100;
nbapp=150;
nbval=50;
nbtest=500;
sigmas=1;
nbtasks=4;
C=100;



rand('seed',0);
randn('seed',0);




verbose=1;


%------------------------------------------------------
% choosing the stopping criterion
%------------------------------------------------------
options.stopvariation=1; % use variation of weights for stopping criterion
options.stopKKT=1;       % set to 1 if you use KKTcondition for stopping criterion


%------------------------------------------------------
% choosing the stopping criterion value
%------------------------------------------------------
options.seuildiffsigma=1e-4;        % stopping criterion for weight variation
options.seuildiffconstraint=0.1;    % stopping criterion for KKT


%------------------------------------------------------
% Setting some numerical parameters
%------------------------------------------------------
options.goldensearch_deltmax=1e-1; % initial precision of golden section search
options.numericalprecision=1e-8;   % numerical precision weights below this value
% are set to zero
options.lambdareg = 1e-8;          % ridge added to kernel matrix

%------------------------------------------------------
% some algorithms paramaters
%------------------------------------------------------
options.firstbasevariable='first'; % tie breaking method for choosing the base
% variable in the reduced gradient method
options.nbitermax=100;             % maximal number of iteration
options.seuil=0;                   % forcing to zero weights lower than this
options.seuilitermax=10;           % value, for iterations lower than this one

options.miniter=0;                 % minimal number of iterations
options.verbosesvm=0;              % verbosity of inner svm algorithm
options.multiclass=0;


%--------------------------------------------------------------------------
% p norm parameter
%--------------------------------------------------------------------------
options.stopvariationlp=1; % use variation of weights for stopping criterion
options.seuildiffsigmalp=1e-2;
options.linear=0;

options.lpnbitermax = 100;
options.norm_p  = 1;
options.epsilon=1;

%
% ------------------------------------------------------------------------
%%                   Building the kernels parameters
%------------------------------------------------------------------------

kernelt={'poly'  };
kerneloptionvect={ [1] };
variablevec={'single'};
dim=N;
[kernel,kerneloptionvec,variableveccell]=CreateKernelListWithVariable(variablevec,dim,kernelt,kerneloptionvect);
[InfoKernel]=CreateInfoKernel(variablevec,dim,kernel,kerneloptionvec);
[X,Y,Xte,Yte,Xval,Yval]=GetMultiTaskFeatures(nbtasks,r,N,nbapp,nbtest,sigmas,nbval);
for k=1:nbtasks
    [X{k},Xval{k},meanxapp{k},stdxapp{k}] = normalizemeanstd(X{k},Xval{k});
    [aux,Xte{k}] = normalizemeanstd(X{k},Xte{k},meanxapp{k},stdxapp{k});
end;
%% Parametres du mklmtl
K=mtlkernels(X,InfoKernel,options);
[K,Weigths]=mtlNormalizekernels(K);


%
% %%-------------------------------------------------------------------------
% %                   Gradient Descent L1-L2
% % %--------------------------------------------------------------------------

tic
 [Sigma,Alpsup,w0,pos,history,obj,status] = mtlmklsvm(K,Y,C,options,verbose);
 toc
 Err0=EvalMultiTask(X,Xte,Yte,InfoKernel,Weigths,Sigma,Alpsup,w0,pos,options);


%
% % %%-------------------------------------------------------------------------
% % %                   Gradient Descent Lp-L2
% % %--------------------------------------------------------------------------
% options.annealing=0;
% [Sigma,Alpsup,w0,pos,history,obj,status,betas] = lpmklmtlsvm(K,Y,C,options,verbose);
% SparsL1=sum(Sigma>1e-5)
%  Err1=EvalMultiTask(X,Xte,Yte,InfoKernel,Weigths,Sigma./betas'.^2,Alpsup,w0,pos,options);
% %


%-------------------------------------------------------------------------
%                  EM L1-Lq
%--------------------------------------------------------------------------
options.norm_q=0;
tic
[d,w,bsvm,posw,Norme,storyEM]=mtlmklsvmEM(K,Y,C,options,verbose);
toc
ErrEM0=EvalMultiTaskEM(X,Xte,Yte,InfoKernel,Weigths,d,w,bsvm,posw,options);
%
%
% %-------------------------------------------------------------------------
% %                  EM Lp-Lq
% %--------------------------------------------------------------------------
options.norm_q=0;
options.annealing=0;
options.epsilon=1e-3;

tic
[d,w,bsvm,posw,NormeFtaskKernel,betasEM] = lpmklmtlsvmEM(K,Y,C,options,verbose);

ErrEM1=EvalMultiTaskEM(X,Xte,Yte,InfoKernel,Weigths,d./(betasEM.^2),w,bsvm,posw,options);
toc
%
%
