function [Sigma,Alpsup,w0,pos,history,obj,status,betas] = lpmklmtlsvm(K,Y,C,options,verbose)

% USAGE [Sigma,Alpsup,w0,pos,history,obj,status] = mklsvm(K,yapp,C,option,verbose)
%
% Inputs
%
% K         : NxNxD matrixs containing all the Gram Matrix for each task in
% a cell
% C         : SVM hyperparameter
% option    : mkl algorithm hyperparameter
%
%       option.nbitermax : maximal number of iterations (default 1000)
%       option.norm_p    : norm used for sparsity 0<p<1
%       option.lpnbitermax: max iteration for obtaining the lp norm..
%       option.algo      : selecting algorithm svmclass (default) or svmreg
%       option.seuil     : threshold for zeroing kernel coefficients
%                          (default 1e-12) during the first
%                          option.seuilitermax
%       option.seuilitermax : threshold weigth when the nb of iteration is
%                           lower than this value
%       option.sigmainit  : initial kernel coefficient (default average)
%       option.alphainit : initial Lagrangian coefficient
%       option.lambdareg : ridge regularization added to kernel's diagonal
%                          for SVM (default 1e-10)
%       option.verbosesvm : verbosity of SVM (default 0) see svmclass or
%                           svmreg
%       option.svmreg_epsilon : epsilon for SVM regression (mandatory for
%                         svm regression)
%       option.numericalprecision    : force to 0 weigth lower than this
%                                     value (default = eps)
%
% Outputs
%
% Sigma         : the weigths
% Alpsup        : the weigthed lagrangian of the support vectors
% w0            : the bias
% pos           : the indices of SV
% history        : history of the weigths
% obj           : objective value
% status        : output status (sucessful or max iter)
if ~isfield(options,'betasinit')
    betas=ones(size(K{1},3),1);
    if ~options.linear
    betas=ones(size(K{1},3),1);
else
    betas=ones(size(K{1},2),1);
end;
    Kb=K;
else
    betas=options.betasinit;
    Kb=mtlreweigthedK(K,betas);
end;

if ~options.linear
    Sigmaold=zeros(1,size(K{1},3));
else
    Sigmaold=zeros(1,size(K{1},2));
end;
        
nbkernel=size(K{1},3);

normp=options.norm_p;

fprintf('Norm p MKLMTL:\n');
fprintf('Norm p = %f \n',normp);

betasold=betas;

loop=1;
nbloop=1;
history.sbeta=[];
history.ssigma=[];
epsilon=options.epsilon;
while epsilon>1e-8;


while loop==1
   
    fprintf('Norm p MKLMTL: Iteration %d\n',nbloop);
    
    [Sigma,Alpsup,w0,pos,storyaux,obj,status,aux] = mtlmklsvm(Kb,Y,C,options,verbose);
    
    
    fprintf('Norm p MKLMTL: Sparsity = %d/%d\n',sum(Sigma~=0),nbkernel);
    [betas]=mtlgetnewbetas(K,Alpsup,Sigma,pos,normp,betas,epsilon);
    
    history.sbeta=[history.sbeta betas];
    history.ssigma=[history.ssigma Sigma];
    
    
    options.sigmainit=Sigma;
    option.alphainit=mtlgetalphainits(K,Alpsup,pos);

    
    nbloop=nbloop+1;
    
    if normp==1
        loop=0;
        fprintf('Norm p MKLMTL: No iteration p=1\n');
    end;
    
    % test loopnumber
    if nbloop>options.lpnbitermax
        loop=0;
        fprintf('Norm p MKLMTL: Maximum iteration number reached\n');
    end
    
    %test sigma variation
    if options.stopvariation==1 && max(abs(Sigma - Sigmaold))<options.seuildiffsigmalp
        loop=0;
        fprintf('Norm p MKLMTL: variation criteria reached\n');
    end
    
    
    
    if loop==1
        Kb=mtlreweigthedK(K,betas);
    end;
    
    Sigmaold=Sigma;
    
    
end
if options.annealing==0
    break
else
epsilon=epsilon/10;
    [betas]=mtlgetnewbetas(K,Alpsup,Sigma,pos,normp,betas,epsilon);
            Kb=mtlreweigthedK(K,betas);
loop=1;
end;
end;

