function [d,w,bsvm,posw,NormeFtaskKernel,weigth,dmat] = lpmklmtlsvmEM(K,Y,C,options,verbose)

% Usage 
%
% 
% [d,w,bsvm,posw,NormeFtaskKernel,weigth,dmat] = lpmklmtlsvmEM(K,Y,C,options,verbose)
%
%  K : cells of kernel. 1 cell per task
%  Y : cells of outputs 1 cell per task
%  C : regularization parameters
%  options.norm_q : this is actually s in the paper.  
%                   should be equal to zero for l1-ell2
%  options.norm_p : p in the ell_p-ell_q penalty
%
%  options.lpnbitermax : max nb of iteration in the reweighted scheme
%  options.seuildiffsigmalp : stop when the weight's variation in the
%                           reweighted scheme is below this value
%
%
%
% outputs
%
% d : the weights
% w,bsvm,posw : usual output stuffs of SVM 1 cell per task
% weight : the beta weights in the paper. You need these weights
% for appropriate evaluation.
%
%
%

% A. Rakotomamonjy 13/08/2010


nbtasks=length(Y);
if options.linear == 0
    nbkernel=size(K{1},3);
else
    nbkernel=size(K{1},2);
end;
%nbkernel=size(K{1},3);
p=options.norm_p ;
q=options.norm_q;
weigth=ones(nbkernel,1);
if ~isfield(options,'sigmainit');
    d=ones(nbtasks,nbkernel)/nbtasks/nbkernel;
else
    d=options.sigmainit;
end;
if ~isfield(options,'multiclass')
    options.multiclass=0;
end;
Kb=K;
eps=options.epsilon;
MatWeigth=[];
dmat=[];
kk=1;
while   eps > 1e-5;
    iter=1;
    dold=d+1;

    while iter <= options.lpnbitermax &  norm(abs(d-dold),'inf')> options.seuildiffsigmalp

        dold=d;
        [d,w,bsvm,posw,NormeFtaskKernelAux,history]=mtlmklsvmEM(Kb,Y,C,options,verbose);
      %  norm(abs(d-dold),'inf')
        for t=1:nbtasks
            for k=1:nbkernel
                if options.linear ==0

                    if options.multiclass==1
                        NormeFtaskKernel(t,k)=d(t,k)*sqrt(abs(w{t}'*K{1}(posw{t},posw{t},k)*w{t}));
                    else
                        NormeFtaskKernel(t,k)=d(t,k)*sqrt(abs(w{t}'*K{t}(posw{t},posw{t},k)*w{t}))/(weigth(k).^2);
                    end;
                else
                    if options.multiclass==1
                        aux=K{1}(posw{t},k)*K{t}(posw{t},k)';
                        NormeFtaskKernel(t,k)=d(t,k)*sqrt(abs(w{t}'*aux*w{t}))/(weigth(k).^2);
                    else
                        aux=K{t}(posw{t},k)*K{t}(posw{t},k)';
                        NormeFtaskKernel(t,k)=d(t,k)*sqrt(abs(w{t}'*aux*w{t}))/(weigth(k).^2);
                    end;


                end;
            end;
        end;
        for k=1:nbkernel
            normeK(k)= sum(NormeFtaskKernel(:,k).^(2/(q+1))).^((q+1)/2);
        end;
        if p<1
            weigth=1./(abs(normeK)+eps).^(1-p);
            MatWeigth=[MatWeigth weigth'];
            Kb=mtlreweigthedK(K,weigth);
            options.sigmainit=d;
            options.alphainit=mtlgetalphainits(K,w,posw,options);
            iter=iter+1;
        else
            weigth=1./(abs(normeK)+eps).^(1-p);
            iter=options.lpnbitermax;
        end;
    end;
    if options.annealing==0
        break
    end
    eps=eps/10;
    weigth=1./(abs(normeK)+eps).^(1-p);
    MatWeigth=[MatWeigth weigth'];
    Kb=mtlreweigthedK(K,weigth);
    dmat{kk}=d;
    kk=kk+1;
    %options=rmfield(options,'sigmainit');
end;
weigth=ones(nbtasks,1)*weigth;