function [d,w,bsvm,posw,NormeFtaskKernel,history]=mtlmklsvmEM(K,Y,C,option,verbose)

% Usage 
%
% [d,w,bsvm,posw,NormeFtaskKernel,history]=mtlmklsvmEM(K,Y,C,option,verbose)
%
%  K : cells of kernel. 1 cell per task
%  Y : cells of outputs 1 cell per task
%  C : regularization parameters
%  options.norm_q : this is actually s in the paper.  
%                   should be equal to zero for l1-ell2
%
%
% outputs
%
% d : the weights
% w,bsvm,posw : usual output stuff of SVM 1 cell per task
%
%
%

% A. Rakotomamonjy 13/08/2010


verbose=option.verbosesvm;
span=1;
nbtasks=length(Y);
if option.linear == 0
    nbkernel=size(K{1},3);
else
    nbkernel=size(K{1},2);
end;



alphazero=zeros(size(K{1},2),1);
q=option.norm_q; % q === s
if ~isfield(option,'sigmainit');
    d=ones(nbtasks,nbkernel);
    d=d./sum((sum(d.^(1/q))).^q);
    if q==0
        d=ones(nbtasks,nbkernel)/nbkernel;
    end
else
    d=option.sigmainit;
end;
if isfield(option,'alphainit');
    alphainit=option.alphainit;
else
    for t=1:nbtasks
        alphainit{t}=[];
    end;
end;
if ~isfield(option,'multiclass')
    option.multiclass=0;
end;
tic
dold=d+1;
nbiter=1;
history.obj=[];
history.time=[];
while norm(d-dold,'inf')> option.seuildiffsigma & nbiter<option.nbitermax
    coutiter=0;
    for t=1:nbtasks
        if option.linear ==0

            if option.multiclass==1
                kerneloption.matrix=sumKbeta(K{1},d(t,:));
            else
                kerneloption.matrix=sumKbeta(K{t},d(t,:));
            end;
        else
            if option.multiclass==1
                kerneloption.matrix=sumXbeta(K{1},d(t,:));
            else
                kerneloption.matrix=sumXbeta(K{t},d(t,:));
            end;
        end;

        [xsup{t},w{t},bsvm{t},posw{t},timeps,alphaw,obj]=svmclass([],Y{t},C,option.lambdareg,'numerical',kerneloption,verbose,span,alphainit{t});
        for i=1:nbkernel
            if option.linear ==0
                if option.multiclass==1
                    NormeFtaskKernel(t,i)=d(t,i)*sqrt(abs(w{t}'*K{1}(posw{t},posw{t},i)*w{t}));
                else
                    NormeFtaskKernel(t,i)=d(t,i)*sqrt(abs(w{t}'*K{t}(posw{t},posw{t},i)*w{t}));
                end;

            else
                aux=K{t}(posw{t},i)*K{t}(posw{t},i)';
                NormeFtaskKernel(t,i)=d(t,i)*sqrt(abs(w{t}'*aux*w{t}));
            end;
        end;
        alphazero=zeros(size(Y{t}));
        alphaaux=alphazero;alphaaux(posw{t})=alphaw;
        alphainit{t}=alphaaux;
        coutiter = coutiter+ obj;

    end
    history.obj=[history.obj coutiter];
    history.time=[history.time toc];

    if q ~=0
        sell= sum(NormeFtaskKernel.^(2/(q+1)));
        sumS= 1./sum( sell.^((q+1)/2));

        % recomputing d using the update equations
        dold=d;
        for t=1:nbtasks
            d(t,:)= NormeFtaskKernel(t,:).^(2*q/(q+1)).*sell.^((1-q)/2)*sumS;
        end;
    else

        sell= sum(NormeFtaskKernel.^2);
        dold=d;
        if nbtasks>1
            for t=1:nbtasks
                d(t,:)= sqrt(sell)./sum(sqrt(sell));
            end;
        else
            d=NormeFtaskKernel/sum(NormeFtaskKernel);
        end;


    end;

    nbiter=nbiter+1;
    % normalization of d should be guaranteed by the updates equation
    % unless q is to small... in those case numerical error may occur
    % using may also result in non-decreasing cost function
    %d=d./sum(sum((d).^(1/q)).^q);

    %     %
    %     % debug
    %     sum(sum(NormeFtaskKernel.^2/d))
    %     sum(sell.^((q+1)/2)).^2
    %


end;
%NormeFtaskKernel

% sum(sum(NormeFtaskKernel.^2/d))
%   sum(sell.^((q+1)/2)).^2
