function [W,b,obj]=mtlmklsvmProxAdapt(X,Y,lambda,options,verbose);

% Usage
%
% [W,b,obj]=mtlmklsvmProx(X,Y,lambda,options,verbose);

%  X : cells of input vector. 1 cell per task
%  Y : cells of outputs 1 cell per task
%  lambda : vector of regularization parameters. lambda is a vector of
%           the size of the number of kernels
%  options.norm_q :    should be equal to 2 for l1-ell2
%  options.seuildiffsigma : stopping criterion on variation of objective
%                               value
%  options.nbitermax : maximal number of iteration
%
%   note that we have implemented a version which X{k} are supposed to have the same dimension
%
%
%  outputs
%
%
%   W  : matrix of weights for linear decision functions
%   b  : bias
%   obj : objective value
%
%
%
%
%

% A. Rakotomamonjy 13/08/2010

nbtasks=length(Y);
dim=size(X{1},2);
if length(lambda)==1
    lambda=lambda*ones(dim,1);
end
if isfield(options,'Winit')
    W=options.Winit;
else
    W=zeros(dim,nbtasks);
end;

V=W;
Wold=W+1;
az=1;
pas=0.001;
eta=0.5;
j=0;
T=3;

for k=1:nbtasks
    b(k)=0;
end;
%while max(max(abs(W-Wold)))>options.seuildiffsigma & j < options.nbitermax

while j< T+1 | (max(obj(j-T:j))-min(obj(j-T:j)))/max(obj(j-T:j))> 0.01;
    normeW=(sum(abs(W).^options.norm_q,2).^(1/options.norm_q))   ;    % shrink

    sumnorm = sum(lambda.*normeW);
    grad=zeros(dim,nbtasks);
    b=findBiasProx(X,Y,W);
    for k=1:nbtasks

        Loss(:,k)=1-Y{k}.*(X{k}*W(:,k)+b(k));
        Loss(:,k)=Loss(:,k).*(Loss(:,k)>0);
        for i=1:size(Loss,1);
            if Loss(i,k) >0
                grad(:,k)=grad(:,k)-(X{k}(i,:)*Y{k}(i))';
            end;
        end;
    end;
    fobj=sum(sum(Loss));

    %--------------------------------------------------------------------
    % Logistic Losss
    %--------------------------------------------------------------------

    %     for k=1:nbtasks
    %         Loss(:,k)=log(1 + exp(-Y{k}.*(X{k}*W(:,k)+b(k))));
    %         for i=1:size(Loss,1);
    %             grad(:,k)=grad(:,k)- (X{k}(i,:)*Y{k}(i)*exp(-Y{k}(i).*(X{k}(i,:)*W(:,k))))' /(1 + exp(-Y{k}(i).*(X{k}(i,:)*W(:,k)+b(k))));
    %
    %         end;
    %     end;
    %     fobj=sum(sum(Loss));


    F=fobj+sumnorm;
    obj(j+1)=F;
    lin=trace((W-V)'*(grad));
    while F > fobj+sumnorm+lin + 1/2/pas*norm(W-V,'fro').^2;
        pas=pas*eta;
    end;
    Wold=W;
    W=V-pas*grad;   % gradient
    switch options.norm_q
        case 2
            for k=1:dim
                normeW=sqrt(sum(W(k,:).^2,2));      % shrink
                bool=normeW>(lambda(k)*pas);
                if bool
                    W(k,:)=(1-lambda(k)*pas./normeW).*W(k,:);
                else
                    W(k,:)=0;
                end;
            end;% shrink
        case 1
            W=(abs(W)-lambda*pas).*((abs(W)-lambda*pas)>0).*sign(W); % l1 solution

        otherwise
            if options.norm_q >2 || options.norm_q<1
                error('this q norm is not handled');
            else
                qp=1/(1-1/options.norm_q);
                opt=[];
                opt.threshold=options.seuildiffsigma;
                for k=1:dim
                    opt.init=W(k,:)';
                    if norm(W(k,:),qp)<lambda(k)*pas
                        W(k,:)=0;
                    else
                        W(k,:)=LSquareLq(W(k,:)',options.norm_q,lambda(k)*pas,opt)';
                    end;
                end;
            end;
    end;



    a1=2/( (j-1)+3);
    delta=W-Wold;
    V=W+(1-az)/az*a1*delta;
    az=a1;
    j=j+1;
    %max(max(abs(W-Wold)))
    %     if j >T+1
    %     (max(obj(j-T:j))-min(obj(j-T:j)))/max(obj(j-T:j))
    %     end;
end;

b=findBiasProx(X,Y,W);




