%
% script for computing the computational complexity
% of the gradient descent approach and the alternate
% optimization approach
%
%
% see Figure 1 of the paper

clear all
close all
addpath('..')
addpath('../grad/')




r=4;                % nb of relevant variables
N=100;              % dimension
nbapp=100;
nbval=100;
nbtest=500;
sigmas=1;           % noise in data
nbtasks=4;          % nb of tasks
C=100;
nbiter=10;
randominit=1;
q= 2; 
if randominit==0
    comment='randominit';
    else
    comment='';
end;

doSave=0;
if doSave
    filename=['Time' '-rel' int2str(r) '-dim' int2str(N) '-nbtask' int2str(nbtasks)...
        '-nbapp' int2str(nbapp) '-q' num2str(q)  comment  ]
end;

 rand('state',0);
 randn('state',0);

verbose=1;

Toyfileoption;


%------------------------------------------------------------------------
%                   Building the kernels parameters
%------------------------------------------------------------------------

kernelt={'poly'  };
kerneloptionvect={ [1] };
variablevec={'single'};


dim=N;

for i=1:nbiter

    [kernel,kerneloptionvec,variableveccell]=CreateKernelListWithVariable(variablevec,dim,kernelt,kerneloptionvect);
    [InfoKernel]=CreateInfoKernel(variablevec,dim,kernel,kerneloptionvec);
    [X,Y,Xte,Yte,Xval,Yval]=GetMultiTaskFeatures(nbtasks,r,N,nbapp,nbtest,sigmas,nbval);

    for k=1:nbtasks
        [X{k},Xval{k},meanxapp{k},stdxapp{k}] = normalizemeanstd(X{k},Xval{k});
        [aux,Xte{k}] = normalizemeanstd(X{k},Xte{k},meanxapp{k},stdxapp{k});
    end;

    %% Parametres du mklmtl

    K=mtlkernels(X,InfoKernel,options);
    [K,Weigths]=mtlNormalizekernels(K);
    nbkernel=size(K{1},3);

    %
    % %%-------------------------------------------------------------------------
    % %                   Gradient Descent L1-L2
    % %--------------------------------------------------------------------------
    

    %
    % if no options.sigmainit then uniform initialisation
    if randominit
    aux=rand(1,nbkernel);
    options.sigmainit=aux./sum(aux);
    end;
    
    tic
    [Sigma,Alpsup,w0,pos,history,obj,status] = mtlmklsvm(K,Y,C,options,verbose);
    time(i)=toc;
    sparseGrad(i)=sum(Sigma>0.01);


    %-------------------------------------------------------------------------
    %                  EM L1-Lq
    %--------------------------------------------------------------------------


    
    options.norm_q= (2-q)/q; % this is actually s 
    if randominit
    d=rand(nbtasks,nbkernel);
    options.sigmainit=d./sum((sum(d.^(1/q))).^q);
    end;
    options.linear=0;
    tic
    [d,w,bsvm,posw,Norme,storyEM]=mtlmklsvmEM(K,Y,C,options,verbose);
    timeEM(i)=toc;
    sparseEM(i)=sum(max(d)>0.01);
    
    normeinf(i)=max(max(abs(abs(ones(nbtasks,1)*Sigma)-d)))
    diffobj(i)=abs(obj-storyEM.obj(end))/obj;


    
%
end;
if doSave
save(filename)
end;

h=plot(history.time,history.obj,'bs-',storyEM.time,storyEM.obj,'rx-')
set(h,'LineWidth',2,'MarkerSize',10);
h=legend('Gradient','Alternate');
set(h,'Fonts',16);
set(gcf,'color','white')
set(gca,'Xscale','log','Fonts',16)
%set(gca,'Yscale','log')
grid
xlabel('CPU time (s)','Fonts',16);
ylabel('Objective value','Fonts',16);

mean(time),std(time)
mean(timeEM),std(timeEM)
mean(diffobj),std(diffobj)
mean(normeinf),std(normeinf)


