
%
% script for computing the computational complexity
% of the linear approaches
%
% see table 1 of the paper

clear all
close all
addpath('..')
addpath('../grad/')


r=4;                % nb of relevant variables
N=100;              % dimension
nbapp=100;
nbval=100;
nbtest=500;
sigmas=1;           % noise in data
nbtasks=4;          % nb of tasks
nbiter=10;
randominit=0;
lambda=35; q=4/3, C=0.1; % perfect sparsity
%lambda=35; q=2, C=0.1; % perfect sparsity
if randominit==1
    comment='randominit';
else
    comment='';
end;

doSave=0;
if doSave
    filename=['Time' '-rel' int2str(r) '-dim' int2str(N) '-nbtask' int2str(nbtasks)...
        '-nbapp' int2str(nbapp) '-q' num2str(q)  comment  ]
end;

rand('state',0);
randn('state',0);

verbose=1;

Toyfileoption;


%------------------------------------------------------------------------
%                   Building the kernels parameters
%------------------------------------------------------------------------

kernelt={'poly'  };
kerneloptionvect={ [1] };
variablevec={'single'};


dim=N;

for i=1:nbiter

    [kernel,kerneloptionvec,variableveccell]=CreateKernelListWithVariable(variablevec,dim,kernelt,kerneloptionvect);
    [InfoKernel]=CreateInfoKernel(variablevec,dim,kernel,kerneloptionvec);
    [X,Y,Xte,Yte,Xval,Yval]=GetMultiTaskFeatures(nbtasks,r,N,nbapp,nbtest,sigmas,nbval);

    for k=1:nbtasks
        [X{k},Xval{k},meanxapp{k},stdxapp{k}] = normalizemeanstd(X{k},Xval{k});
        [aux,Xte{k}] = normalizemeanstd(X{k},Xte{k},meanxapp{k},stdxapp{k});
    end;

    %% Parametres du mklmtl

    K=mtlkernels(X,InfoKernel,options);
    [K,Weigths]=mtlNormalizekernels(K);

    %-------------------------------------------------------------------------
    %                  EM L1-Lq
    %--------------------------------------------------------------------------

    options.norm_q= (2-q)/q; % this is actually s
    if randominit
        d=rand(nbtasks,nbkernel);
        options.sigmainit=d./sum((sum(d.^(1/q))).^q);
    end;
    options.linear=0;
    tic
    [d,w,bsvm,posw,Norme,storyEM]=mtlmklsvmEM(K,Y,C,options,verbose);
    timeEM(i)=toc;
    sparseEM(i)=sum(sum(d)>0.01);




    %-------------------------------------------------------------------------
    %                  EM L1-Lq Linear
    %--------------------------------------------------------------------------

    options.norm_q= (2-q)/q; % this is actually s
    if randominit
        d=rand(nbtasks,nbkernel);
        options.sigmainit=d./sum((sum(d.^(1/q))).^q);
    end;
    options.linear=1;
    tic
    [d,w,bsvm,posw,Norme,storyEM]=mtlmklsvmEM(X,Y,C,options,verbose);
    timeEMlinear(i)=toc;
    sparseEMlinear(i)=sum(max(d)>0.01);


    %-------------------------------------------------------------------------
    %                  L1-Lq Accerelated Gradrient
    %--------------------------------------------------------------------------

    opt.norm_q=q;
    opt.seuildiffsigma=options.seuildiffsigma;
    opt.nbitermax=options.nbitermax;
    tic
    W=mtlmklsvmProx(X,Y,lambda,opt,verbose);
    timeAG(i)=toc;
    sparseAG(i)=sum(sum(abs(W),2)>0.01);

    %
end;


mean(timeEM),std(timeEM)
mean(timeEMlinear),std(timeEMlinear)
mean(timeAG),std(timeAG)

