%Experiments for FelMuG for NCI Graph data
% calling Java for Top-K subgraph mining
% Input--noTrain: number of training graphs
%        seed: a seed number for perform random split of the trainign
%        graphs
%        files: graph task files
%        K: common features
%
%
function [ErrL1L2,accs,AUCL1L2,auc,cp]=demo_FelMuG_graph(noTrain,seed,files,K)

InstallFelMug %adding the classpath

%nargin
t1=cputime;
javasetting
import util.*;
import moss.*;
datapath='./';

if nargin<1
    noTrain=50;
end
if nargin<2
    seed = 2;
end
if nargin <3
    files={'1','33','41','47','81','83','109','123','145'};
end
if nargin < 4
    K=15;
end

T=length(files);


normalizeKernels=0; %normalize multiple kernels each time before training?
opt.maxInt = 15; %maximum number of iterations
opt.K=K; %Least Number of Features for Multi-task learning, default 15 for NCI
opt.epslog=5*1e-3; %stopping condition for outer iterations
opt.C=.1; %K=0.5 %.1 for Graph classification
opt.normalizeKernels = normalizeKernels;
opt.debug_msg=1;
opt.equalmu=0;
opt.files=files;

tS=1; %1 for experiment
Kt = tS * ones(T,1); 
aK=2; %1 for experiment; 2 for varying K experiments
Ka=aK * ones(T,1);
opt.Ka=Ka;

opt.Kt=Kt;
opt.maxPad=8;
opt.supp=0;




T=length(files);
train_G=cell(T,1);
test_G=cell(T,1);
train_Y=cell(T,1);
test_Y=cell(T,1);
for i = 1:length(files)
    disp(['Reading task',num2str(i),'.....'])
    
    %% Calling Java to create trainning graphs
    graphs=GraphUtils.readGraphs([datapath, '/NCI_data/NCI_balanced/',files{i},'-balance.sdf'], 0);% NCI
    split = TrainTestSplit;
    split.setSeed(seed);
  %  split.split(graphs,noTrain); %for protein, PTC

    split.splitbyNo(graphs,noTrain); % for NCI, only select noTrain graphs

    train_G{i} = split.trainG;
    test_G{i} = split.testG;
    train_Y{i}=split.trainY;
    test_Y{i}=split.testY;



    no_p = nnz(train_Y{i}==1);
    no_n = nnz(train_Y{i}==-1);
    disp(['pos:',num2str(no_p),'  neg:',num2str(no_n)])
    

end



 %% Calling FelMuG algorithm
 [model] = FelMuG_graph(train_G,train_Y,opt);
 disp('Training over...')
 
 % Evaluation
 [ErrL1L2,ypred,AUCL1L2,accs,auc]=EvalMultiTask_Graph(test_G,test_Y,model);
% toc
 disp(['ACC:',num2str(1-ErrL1L2),' AUC:',num2str(AUCL1L2)])
 cp=cputime-t1;
